@@ -1179,15 +1179,28 @@ static void widget_set_cursor(GtkWidget *widget, gint icon)
  *-------------------------------------------------------------------
  */
 
-static void image_zoom_sync(ImageWindow *imd, gfloat zoom, gint force, gint blank, gint new)
+static void image_zoom_sync(ImageWindow *imd, gfloat zoom,
+			    gint force, gint blank, gint new,
+			    gint center_point, gint px, gint py)
 {
 	gfloat old_scale;
 	gint old_cx, old_cy;
 	gint sized;
 
 	old_scale = imd->scale;
-	old_cx = imd->x_scroll + imd->vis_width / 2;
-	old_cy = imd->y_scroll + imd->vis_height / 2;
+	if (center_point)
+		{
+		px = CLAMP(px, 0, imd->width);
+		py = CLAMP(py, 0, imd->height);
+		old_cx = imd->x_scroll + (px - imd->x_offset);
+		old_cy = imd->y_scroll + (py - imd->y_offset);
+		}
+	else
+		{
+		px = py = 0;
+		old_cx = imd->x_scroll + imd->vis_width / 2;
+		old_cy = imd->y_scroll + imd->vis_height / 2;
+		}
 
 	if (!image_zoom_clamp(imd, zoom, force, new)) return;
 
@@ -1204,8 +1217,16 @@ static void image_zoom_sync(ImageWindow *imd, gfloat zoom, gint force, gint blan
 	if (!force)
 		{
 		/* user zoom does not force, so keep visible center point */
-		imd->x_scroll = old_cx / old_scale * imd->scale - (imd->vis_width / 2);
-		imd->y_scroll = old_cy / old_scale * imd->scale - (imd->vis_height / 2);
+		if (center_point)
+			{
+			imd->x_scroll = old_cx / old_scale * imd->scale - (px - imd->x_offset);
+			imd->y_scroll = old_cy / old_scale * imd->scale - (py - imd->y_offset);
+			}
+		else
+			{
+			imd->x_scroll = old_cx / old_scale * imd->scale - (imd->vis_width / 2);
+			imd->y_scroll = old_cy / old_scale * imd->scale - (imd->vis_height / 2);
+			}
 		}
 	image_scroll_clamp(imd);
 
@@ -1243,7 +1264,7 @@ static void image_pixbuf_sync(ImageWindow *imd, gfloat zoom, gint blank, gint ne
 	imd->x_scroll = 0;
 	imd->y_scroll = 0;
 
-	image_zoom_sync(imd, zoom, TRUE, blank, new);
+	image_zoom_sync(imd, zoom, TRUE, blank, new, FALSE, 0, 0);
 }
 
 static void image_set_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gfloat zoom, gint new)
@@ -1258,27 +1279,42 @@ static void image_set_pixbuf(ImageWindow *imd, GdkPixbuf *pixbuf, gfloat zoom, g
 static void image_alter_real(ImageWindow *imd, AlterType type)
 {
 	GdkPixbuf *new = NULL;
+	gint x, y;
+	gint t;
 
 	imd->delay_alter_type = ALTER_NONE;
 
 	if (!imd->pixbuf) return;
 
+	x = imd->x_scroll + (imd->vis_width / 2);
+	y = imd->y_scroll + (imd->vis_height / 2);
+
 	switch (type)
 		{
 		case ALTER_ROTATE_90:
 			new = pixbuf_copy_rotate_90(imd->pixbuf, FALSE);
+			t = x;
+			x = imd->height - y;
+			y = t;
 			break;
 		case ALTER_ROTATE_90_CC:
 			new = pixbuf_copy_rotate_90(imd->pixbuf, TRUE);
+			t = x;
+			x = y;
+			y = imd->width - t;
 			break;
 		case ALTER_ROTATE_180:
 			new = pixbuf_copy_mirror(imd->pixbuf, TRUE, TRUE);
+			x = imd->width - x;
+			y = imd->height - y;
 			break;
 		case ALTER_MIRROR:
 			new = pixbuf_copy_mirror(imd->pixbuf, TRUE, FALSE);
+			x = imd->width - x;
 			break;
 		case ALTER_FLIP:
 			new = pixbuf_copy_mirror(imd->pixbuf, FALSE, TRUE);
+			y = imd->height - y;
 			break;
 		case ALTER_NONE:
 		default:
@@ -1290,6 +1326,11 @@ static void image_alter_real(ImageWindow *imd, AlterType type)
 		{
 		image_set_pixbuf(imd, new, imd->zoom, TRUE);
 		gdk_pixbuf_unref(new);
+
+		if (imd->zoom != 0.0)
+			{
+			image_scroll(imd, x - (imd->vis_width / 2), y - (imd->vis_height / 2));
+			}
 		}
 }
 
@@ -1705,9 +1746,9 @@ static gint image_focus_in_cb(GtkWidget *widget, GdkEventFocus *event, gpointer
 	ImageWindow *imd = data;
 
 	GTK_WIDGET_SET_FLAGS(imd->widget, GTK_HAS_FOCUS);
+	gtk_frame_set_shadow_type(GTK_FRAME(imd->widget), GTK_SHADOW_NONE);
 
-	gtk_widget_queue_draw(imd->widget);
-	return FALSE;
+	return TRUE;
 }
 
 static gint image_focus_out_cb(GtkWidget *widget, GdkEventFocus *event, gpointer data)
@@ -1715,9 +1756,9 @@ static gint image_focus_out_cb(GtkWidget *widget, GdkEventFocus *event, gpointer
 	ImageWindow *imd = data;
 
 	GTK_WIDGET_UNSET_FLAGS(imd->widget, GTK_HAS_FOCUS);
-	gtk_widget_queue_clear(imd->widget);
 	gtk_frame_set_shadow_type(GTK_FRAME(imd->widget), GTK_SHADOW_IN);
-	return FALSE;
+
+	return TRUE;
 }
 
 /*
@@ -2020,6 +2061,11 @@ void image_change_from_image(ImageWindow *imd, ImageWindow *source)
 
 	imd->read_ahead_path = source->read_ahead_path;
 	source->read_ahead_path = NULL;
+
+	imd->x_scroll = source->x_scroll;
+	imd->y_scroll = source->y_scroll;
+
+	image_scroll_clamp(imd);
 }
 
 /* manipulation */
@@ -2048,7 +2094,8 @@ void image_alter(ImageWindow *imd, AlterType type)
 
 /* zoom */
 
-void image_zoom_adjust(ImageWindow *imd, gfloat increment)
+static void image_zoom_adjust_real(ImageWindow *imd, gfloat increment,
+				   gint center_point, gint x, gint y)
 {
 	gfloat zoom = imd->zoom;
 
@@ -2056,19 +2103,17 @@ void image_zoom_adjust(ImageWindow *imd, gfloat increment)
 
 	if (zoom == 0.0)
 		{
-		gfloat new_zoom;
 		if (imd->scale < 1.0)
 			{
-			new_zoom = 0.0 - 1.0 / imd->scale;
+			zoom = 0.0 - 1.0 / imd->scale;
 			}
 		else
 			{
-			new_zoom = imd->scale;
+			zoom = imd->scale;
 			}
-		/* simply use the increment as the step size and lock (round) the new zoom to it */
-		zoom = ceil(new_zoom / increment) * increment;
 		}
-	else if (increment < 0.0)
+
+	if (increment < 0.0)
 		{
 		if (zoom >= 1.0 && zoom + increment < 1.0)
 			{
@@ -2091,12 +2136,22 @@ void image_zoom_adjust(ImageWindow *imd, gfloat increment)
 			}
 		}
 
-	image_zoom_sync(imd, zoom, FALSE, FALSE, FALSE);
+	image_zoom_sync(imd, zoom, FALSE, FALSE, FALSE, center_point, x, y);
+}
+
+void image_zoom_adjust(ImageWindow *imd, gfloat increment)
+{
+	image_zoom_adjust_real(imd, increment, FALSE, 0, 0);
+}
+
+void image_zoom_adjust_at_point(ImageWindow *imd, gfloat increment, gint x, gint y)
+{
+	image_zoom_adjust_real(imd, increment, TRUE, x, y);
 }
 
 void image_zoom_set(ImageWindow *imd, gfloat zoom)
 {
-	image_zoom_sync(imd, zoom, FALSE, FALSE, FALSE);
+	image_zoom_sync(imd, zoom, FALSE, FALSE, FALSE, FALSE, 0, 0);
 }
 
 gfloat image_zoom_get(ImageWindow *imd)
