@@ -15,12 +15,14 @@
 
 #include "collect.h"
 #include "dnd.h"
+#include "filelist.h"
 #include "fullscreen.h"
 #include "image.h"
 #include "info.h"
 #include "menu.h"
 #include "slideshow.h"
 #include "utilops.h"
+#include "ui_fileops.h"
 #include "ui_menu.h"
 
 #include <gdk/gdkkeysyms.h> /* for keyboard values */
@@ -35,6 +37,8 @@ struct _ViewWindow
 	ImageWindow *imd;
 	FullScreenData *fs;
 	SlideShowData *ss;
+
+	GList *list;
 };
 
 
@@ -51,6 +55,8 @@ static void view_slideshow_stop(ViewWindow *vw);
 
 static void view_window_close(ViewWindow *vw);
 
+static void view_window_dnd_init(ViewWindow *vw);
+
 
 /*
  *-----------------------------------------------------------------------------
@@ -65,6 +71,34 @@ static ImageWindow *view_window_active_image(ViewWindow *vw)
 	return vw->imd;
 }
 
+static GList *path_list_copy(GList *list)
+{
+	GList *new_list = NULL;
+	GList *work;
+
+	work = list;
+	while (work)
+		{
+		gchar *path;
+
+		path = work->data;
+		work = work->next;
+
+		new_list = g_list_prepend(new_list, g_strdup(path));
+		}
+
+	return g_list_reverse(new_list);
+}
+
+static void view_window_set_list(ViewWindow *vw, GList *list)
+{
+
+	path_list_free(vw->list);
+	vw->list = NULL;
+
+	vw->list = path_list_copy(list);
+}
+
 static gint view_window_contains_collection(ViewWindow *vw)
 {
 	CollectionData *cd;
@@ -144,12 +178,101 @@ static void view_collection_step_to_end(ViewWindow *vw, gint last)
 	
 }
 
+static void view_list_step(ViewWindow *vw, gint next)
+{
+	ImageWindow *imd = view_window_active_image(vw);
+	const gchar *path;
+	GList *work;
+	GList *work_ahead;
+	gint found;
+
+	if (!vw->list) return;
+
+	path = image_get_path(imd);
+	if (!path) return;
+
+	found = FALSE;
+	work = vw->list;
+	while (work && !found)
+		{
+		gchar *temp;
+
+		temp = work->data;
+
+		if (strcmp(path, temp) == 0)
+			{
+			found = TRUE;
+			}
+		else
+			{
+			work = work->next;
+			}
+		}
+	if (!work) return;
+
+	work_ahead = NULL;
+	if (next)
+		{
+		work = work->next;
+		if (work) work_ahead = work->next;
+		}
+	else
+		{
+		work = work->prev;
+		if (work) work_ahead = work->prev;
+		}
+	if (!work) return;
+
+	path = work->data;
+	image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
+
+	if (enable_read_ahead && work_ahead)
+		{
+		const gchar *next_path = work_ahead->data;
+		image_prebuffer_set(imd, next_path);
+		}
+}
+
+static void view_list_step_to_end(ViewWindow *vw, gint last)
+{
+	ImageWindow *imd = view_window_active_image(vw);
+	const gchar *path;
+	GList *work;
+	GList *work_ahead;
+
+	if (!vw->list) return;
+
+	if (last)
+		{
+		work = g_list_last(vw->list);
+		work_ahead = work->prev;
+		}
+	else
+		{
+		work = vw->list;
+		work_ahead = work->next;
+		}
+
+	path = work->data;
+	image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
+
+	if (enable_read_ahead && work_ahead)
+		{
+		const gchar *next_path = work_ahead->data;
+		image_prebuffer_set(imd, next_path);
+		}
+}
+
 static void view_step_next(ViewWindow *vw)
 {
 	if (vw->ss)
 		{
 		view_slideshow_next(vw);
 		}
+	else if (vw->list)
+		{
+		view_list_step(vw, TRUE);
+		}
 	else
 		{
 		view_collection_step(vw, TRUE);
@@ -162,12 +285,28 @@ static void view_step_prev(ViewWindow *vw)
 		{
 		view_slideshow_prev(vw);
 		}
+	else if (vw->list)
+		{
+		view_list_step(vw, FALSE);
+		}
 	else
 		{
 		view_collection_step(vw, FALSE);
 		}
 }
 
+static void view_step_to_end(ViewWindow *vw, gint last)
+{
+	if (vw->list)
+		{
+		view_list_step_to_end(vw, last);
+		}
+	else
+		{
+		view_collection_step_to_end(vw, last);
+		}
+}
+
 /*
  *-----------------------------------------------------------------------------
  * view window keyboard
@@ -215,11 +354,11 @@ static gint view_window_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpoi
 			stop_signal = TRUE;
 			break;
 		case GDK_Home: case GDK_KP_Home:
-			view_collection_step_to_end(vw, FALSE);
+			view_step_to_end(vw, FALSE);
 			stop_signal = TRUE;
 			break;
 		case GDK_End: case GDK_KP_End:
-			view_collection_step_to_end(vw, TRUE);
+			view_step_to_end(vw, TRUE);
 			stop_signal = TRUE;
 			break;
 		case '+': case '=': case GDK_KP_Add:
@@ -418,13 +557,11 @@ static void button3_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
 
 static void button4_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
 {
-#if 0
 	ViewWindow *vw = data;
-#endif
 
 	if (bevent->state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust(imd, get_zoom_increment());
+		image_zoom_adjust_at_point(imd, get_zoom_increment(), bevent->x, bevent->y);
 		}
 	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
@@ -432,19 +569,17 @@ static void button4_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
 		}
 	else
 		{
-		/* FIXME: implement moving among image in view windows */
+		view_step_prev(vw);
 		}
 }
 
 static void button5_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
 {
-#if 0
 	ViewWindow *vw = data;
-#endif
 
 	if (bevent->state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust(imd, -get_zoom_increment());
+		image_zoom_adjust_at_point(imd, -get_zoom_increment(), bevent->x, bevent->y);
 		}
 	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
@@ -452,7 +587,7 @@ static void button5_cb(ImageWindow *imd, GdkEventButton *bevent, gpointer data)
 		}
 	else
 		{
-		/* FIXME: implement moving among image in view windows */
+		view_step_next(vw);
 		}
 }
 
@@ -519,6 +654,14 @@ static void view_slideshow_start(ViewWindow *vw)
 		CollectionData *cd;
 		CollectInfo *info;
 
+		if (vw->list)
+			{
+			vw->ss = slideshow_start_from_path_list(view_window_active_image(vw),
+								path_list_copy(vw->list),
+								view_slideshow_stop_func, vw);
+			return;
+			}
+
 		cd = image_get_collection(view_window_active_image(vw), &info);
 		if (cd && info)
 			{
@@ -540,6 +683,7 @@ static void view_window_close(ViewWindow *vw)
 	view_slideshow_stop(vw);
 	view_fullscreen_toggle(vw, TRUE);
 	gtk_widget_destroy(vw->window);
+	path_list_free(vw->list);
 	g_free(vw);
 }
 
@@ -551,17 +695,18 @@ static gint view_window_delete_cb(GtkWidget *w, GdkEventAny *event, gpointer dat
 	return TRUE;
 }
 
-static void real_view_window_new(const gchar *path, CollectionData *cd, CollectInfo *info)
+static void real_view_window_new(const gchar *path, GList *list, CollectionData *cd, CollectInfo *info)
 {
 	ViewWindow *vw;
 	GtkAllocation req_size;
 	gint w, h;
 
-	if (!path && (!cd || !info)) return;
+	if (!path && !list && (!cd || !info)) return;
 
 	vw = g_new0(ViewWindow, 1);
 	vw->fs = NULL;
 	vw->ss = NULL;
+	vw->list = NULL;
 
 	vw->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_policy(GTK_WINDOW(vw->window), TRUE, TRUE, FALSE);
@@ -582,7 +727,7 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 	gtk_container_add(GTK_CONTAINER(vw->window), vw->imd->widget);
 	gtk_widget_show(vw->imd->widget);
 
-	image_dnd_init(vw->imd);
+	view_window_dnd_init(vw);
 
 	view_image_set_buttons(vw, vw->imd);
 
@@ -590,7 +735,6 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 			   GTK_SIGNAL_FUNC(view_window_delete_cb), vw);
 	gtk_signal_connect(GTK_OBJECT(vw->window), "key_press_event",
 			   GTK_SIGNAL_FUNC(view_window_key_press_cb), vw);
-
 	if (cd && info)
 		{
 		image_change_from_collection(vw->imd, cd, info, image_zoom_get_default(NULL, zoom_mode));
@@ -601,6 +745,16 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 			if (r_info) image_prebuffer_set(vw->imd, r_info->path);
 			}
 		}
+	else if (list)
+		{
+		view_window_set_list(vw, list);
+		image_change_path(vw->imd, (gchar *)vw->list->data, image_zoom_get_default(NULL, zoom_mode));
+		if (enable_read_ahead)
+			{
+			GList *work = vw->list->next;
+			if (work) image_prebuffer_set(vw->imd, (gchar *)work->data);
+			}
+		}
 	else
 		{
 		image_change_path(vw->imd, path, image_zoom_get_default(NULL, zoom_mode));
@@ -640,12 +794,17 @@ static void real_view_window_new(const gchar *path, CollectionData *cd, CollectI
 
 void view_window_new(const gchar *path)
 {
-	real_view_window_new(path, NULL, NULL);
+	real_view_window_new(path, NULL, NULL, NULL);
+}
+
+void view_window_new_from_list(GList *list)
+{
+	real_view_window_new(NULL, list, NULL, NULL);
 }
 
 void view_window_new_from_collection(CollectionData *cd, CollectInfo *info)
 {
-	real_view_window_new(NULL, cd, info);
+	real_view_window_new(NULL, NULL, cd, info);
 }
 
 /*
@@ -862,7 +1021,7 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 
 	if (vw->ss)
 		{
-		menu_item_add(menu, _("Stop slideshow"), view_slideshow_stop_cb, vw);
+		menu_item_add(menu, _("Stop slideshow"), GTK_SIGNAL_FUNC(view_slideshow_stop_cb), vw);
 		if (slideshow_paused(vw->ss))
 			{
 			item = menu_item_add(menu, _("Continue slideshow"),
@@ -877,7 +1036,7 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 	else
 		{
 		item = menu_item_add(menu, _("Start slideshow"), GTK_SIGNAL_FUNC(view_slideshow_start_cb), vw);
-		gtk_widget_set_sensitive(item, view_window_contains_collection(vw));
+		gtk_widget_set_sensitive(item, (vw->list != NULL) || view_window_contains_collection(vw));
 		item = menu_item_add(menu, _("Pause slideshow"), GTK_SIGNAL_FUNC(view_slideshow_pause_cb), vw);
 		gtk_widget_set_sensitive(item, FALSE);
 		}
@@ -897,3 +1056,379 @@ static GtkWidget *view_popup_menu(ViewWindow *vw)
 	return menu;
 }
 
+/*
+ *-------------------------------------------------------------------
+ * dnd confirm dir
+ *-------------------------------------------------------------------
+ */
+
+typedef struct {
+	ViewWindow *vw;
+	GList *list;
+} CViewConfirmD;
+
+static void view_dir_list_cancel(GtkWidget *widget, gpointer data)
+{
+	/* do nothing */
+}
+
+static void view_dir_list_do(ViewWindow *vw, GList *list, gint skip, gint recurse)
+{
+	GList *work;
+
+	view_window_set_list(vw, NULL);
+
+	work = list;
+	while (work)
+		{
+		gchar *path = work->data;
+		work = work->next;
+
+		if (isdir(path))
+			{
+			if (!skip)
+				{
+				GList *list = NULL;
+
+				if (recurse)
+					{
+					list = path_list_recursive(path);
+					}
+				else
+					{
+					path_list(path, &list, NULL);
+					list = path_list_sort(list);
+					list = path_list_filter(list, FALSE);
+					}
+				if (list) vw->list = g_list_concat(vw->list, list);
+				}
+			}
+		else
+			{
+			/* FIXME: no filtering here */
+			vw->list = g_list_append(vw->list, g_strdup(path));
+			}
+		}
+
+	if (vw->list)
+		{
+		gchar *path;
+
+		path = vw->list->data;
+		image_change_path(vw->imd, path, image_zoom_get_default(vw->imd, zoom_mode));
+
+		work = vw->list->next;
+		if (enable_read_ahead && work)
+			{
+			path = work->data;
+			image_prebuffer_set(vw->imd, path);
+			}
+		}
+	else
+		{
+		image_change_path(vw->imd, NULL, image_zoom_get_default(vw->imd, zoom_mode));
+		}
+}
+
+static void view_dir_list_add(GtkWidget *widget, gpointer data)
+{
+	CViewConfirmD *d = data;
+	view_dir_list_do(d->vw, d->list, FALSE, FALSE);
+}
+
+static void view_dir_list_recurse(GtkWidget *widget, gpointer data)
+{
+	CViewConfirmD *d = data;
+	view_dir_list_do(d->vw, d->list, FALSE, TRUE);
+}
+
+static void view_dir_list_skip(GtkWidget *widget, gpointer data)
+{
+	CViewConfirmD *d = data;
+	view_dir_list_do(d->vw, d->list, TRUE, FALSE);
+}
+
+static void view_dir_list_destroy(GtkWidget *widget, gpointer data)
+{
+	CViewConfirmD *d = data;
+        path_list_free(d->list);
+        g_free(d);
+}
+
+static GtkWidget *view_confirm_dir_list(ViewWindow *vw, GList *list)
+{
+	GtkWidget *menu;
+	CViewConfirmD *d;
+
+	d = g_new(CViewConfirmD, 1);
+	d->vw = vw;
+	d->list = list;
+
+	menu = popup_menu_short_lived();
+	gtk_signal_connect(GTK_OBJECT(menu), "destroy",
+			   GTK_SIGNAL_FUNC(view_dir_list_destroy), d);
+
+	menu_item_add(menu, _("Dropped list includes directories."), NULL, NULL);
+	menu_item_add_divider(menu);
+	menu_item_add(menu, _("Add contents"), GTK_SIGNAL_FUNC(view_dir_list_add), d);
+	menu_item_add(menu, _("Add contents recursive"), GTK_SIGNAL_FUNC(view_dir_list_recurse), d);
+	menu_item_add(menu, _("Skip directories"), GTK_SIGNAL_FUNC(view_dir_list_skip), d);
+	menu_item_add_divider(menu);
+	menu_item_add(menu, _("Cancel"), GTK_SIGNAL_FUNC(view_dir_list_cancel), d);
+
+	return menu;
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ * image drag and drop routines
+ *-----------------------------------------------------------------------------
+ */
+
+static void view_window_get_dnd_data(GtkWidget *widget, GdkDragContext *context,
+				     gint x, gint y,
+				     GtkSelectionData *selection_data, guint info,
+				     guint time, gpointer data)
+{
+	ViewWindow *vw = data;
+	ImageWindow *imd;
+
+	if (gtk_drag_get_source_widget(context) == vw->imd->image) return;
+
+	imd = vw->imd;
+
+	if (info == TARGET_URI_LIST || info == TARGET_APP_COLLECTION_MEMBER)
+		{
+		CollectionData *source;
+		GList *list;
+		GList *info_list;
+
+		if (info == TARGET_URI_LIST)
+			{
+			GList *work;
+
+			list = get_uri_file_list(selection_data->data);
+
+			work = list;
+			while (work)
+				{
+				if (isdir((gchar *)work->data))
+					{
+					GtkWidget *menu;
+					menu = view_confirm_dir_list(vw, list);
+					gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, time);
+					return;
+					}
+				work = work->next;
+				}
+
+			list = path_list_filter(list, FALSE);
+
+			source = NULL;
+			info_list = NULL;
+			}
+		else
+			{
+			source = collection_from_dnd_data((gchar *)selection_data->data, &list, &info_list);
+			}
+
+		if (list)
+			{
+			gchar *path;
+
+			path = list->data;
+			if (isfile(path))
+				{
+				view_slideshow_stop(vw);
+				view_window_set_list(vw, NULL);
+				if (source && info_list)
+					{
+					image_change_from_collection(imd, source, info_list->data, image_zoom_get_default(imd, zoom_mode));
+					}
+				else
+					{
+					if (list->next)
+						{
+						vw->list = list;
+						list = NULL;
+						}
+					image_change_path(imd, path, image_zoom_get_default(imd, zoom_mode));
+					}
+				}
+			}
+		path_list_free(list);
+		g_list_free(info_list);
+		}
+}
+
+static void view_window_set_dnd_data(GtkWidget *widget, GdkDragContext *context,
+				     GtkSelectionData *selection_data, guint info,
+				     guint time, gpointer data)
+{
+	ViewWindow *vw = data;
+	const gchar *path;
+
+	path = image_get_path(vw->imd);
+
+	if (path)
+		{
+		gchar *text = NULL;
+		switch (info)
+			{
+			case TARGET_URI_LIST:
+				text = g_strconcat("file:", path, "\r\n", NULL);
+				break;
+			case TARGET_TEXT_PLAIN:
+				text = g_strdup(path);
+				break;
+			}
+		if (text)
+			{
+			gtk_selection_data_set (selection_data, selection_data->target,
+						8, text, strlen(text));
+			g_free(text);
+			}
+		}
+	else
+		{
+		gtk_selection_data_set (selection_data, selection_data->target,
+					8, NULL, 0);
+		}
+}
+
+static void view_window_dnd_init(ViewWindow *vw)
+{
+	ImageWindow *imd;
+
+	imd = vw->imd;
+
+	gtk_drag_source_set(imd->image, GDK_BUTTON2_MASK,
+			    dnd_file_drag_types, dnd_file_drag_types_count,
+			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_get",
+			   GTK_SIGNAL_FUNC(view_window_set_dnd_data), vw);
+
+	gtk_drag_dest_set(imd->image,
+			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			  dnd_file_drop_types, dnd_file_drop_types_count,
+                          GDK_ACTION_COPY);
+	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_received",
+			   GTK_SIGNAL_FUNC(view_window_get_dnd_data), vw);
+}
+
+/*
+ *-----------------------------------------------------------------------------
+ * maintenance (for rename, move, remove)
+ *-----------------------------------------------------------------------------
+ */
+
+static void view_real_removed(ViewWindow *vw, const gchar *path, GList *ignore_list)
+{
+	ImageWindow *imd;
+	const gchar *image_path;
+
+	imd = view_window_active_image(vw);
+	image_path = image_get_path(imd);
+
+	if (image_path && strcmp(image_path, path) == 0)
+		{
+		if (vw->list)
+			{
+			view_list_step(vw, TRUE);
+			if (image_get_path(imd) == image_path)
+				{
+				view_list_step(vw, FALSE);
+				}
+			}
+		else if (view_window_contains_collection(vw))
+			{
+			view_collection_step(vw, TRUE);
+			if (image_get_path(imd) == image_path)
+				{
+				view_collection_step(vw, FALSE);
+				}
+			}
+		if (image_get_path(imd) == image_path)
+			{
+			image_change_path(imd, NULL, image_zoom_get_default(imd, zoom_mode));
+			}
+		}
+
+	if (vw->list)
+		{
+		GList *work;
+		work = vw->list;
+		while (work)
+			{
+			gchar *chk_path;
+
+			chk_path = work->data;
+			work = work->next;
+
+			if (strcmp(chk_path, path) == 0)
+				{
+				vw->list = g_list_remove(vw->list, chk_path);
+				g_free(chk_path);
+				}
+			}
+		}
+}
+
+static void view_real_moved(ViewWindow *vw, const gchar *source, const gchar *dest)
+{
+	ImageWindow *imd;
+	const gchar *image_path;
+
+	imd = view_window_active_image(vw);
+	image_path = image_get_path(imd);
+
+	if (image_path && strcmp(image_path, source) == 0)
+		{
+		image_set_path(imd, dest);
+		}
+
+	if (vw->list)
+		{
+		GList *work;
+		work = vw->list;
+		while (work)
+			{
+			gchar *chk_path;
+
+			chk_path = work->data;
+
+			if (strcmp(chk_path, source) == 0)
+				{
+				work->data = g_strdup(dest);
+				g_free(chk_path);
+				}
+
+			work = work->next;
+			}
+		}
+}
+
+void view_window_maint_removed(const gchar *path, GList *ignore_list)
+{
+	GList *work = view_window_list;
+	while (work)
+		{
+		ViewWindow *vw = work->data;
+		work = work->next;
+
+		view_real_removed(vw, path, ignore_list);
+		}
+}
+
+void view_window_maint_moved(const gchar *source, const gchar *dest)
+{
+	GList *work = view_window_list;
+	while (work)
+		{
+		ViewWindow *vw = work->data;
+		work = work->next;
+
+		view_real_moved(vw, source, dest);
+		}
+}
+
