@@ -13,6 +13,7 @@
 #include "gqview.h"
 #include "info.h"
 
+#include "dnd.h"
 #include "filelist.h"
 #include "image.h"
 #include "ui_fileops.h"
@@ -200,6 +201,7 @@ struct _InfoTabGeneral
 	GtkWidget *label_file_time;
 	GtkWidget *label_file_size;
 	GtkWidget *label_dimensions;
+	GtkWidget *label_transparent;
 	GtkWidget *label_image_size;
 	GtkWidget *label_compression;
 	gint compression_done;
@@ -211,6 +213,7 @@ static void info_tab_general_image(InfoData *id, gpointer data)
 	InfoTabGeneral *tab = data;
 	gchar *buf;
 	gint mem_size;
+	gint has_alpha;
 
 	if (id->image->unknown) return;
 
@@ -218,6 +221,16 @@ static void info_tab_general_image(InfoData *id, gpointer data)
 	gtk_label_set(GTK_LABEL(tab->label_dimensions), buf);
 	g_free(buf);
 
+	if (id->image->pixbuf)
+		{
+		has_alpha = gdk_pixbuf_get_has_alpha(id->image->pixbuf);
+		}
+	else
+		{
+		has_alpha = FALSE;
+		}
+	gtk_label_set(GTK_LABEL(tab->label_transparent), has_alpha ? _("yes") : _("no"));
+
 	mem_size = id->image->image_width * id->image->image_width * 3;
 	buf = text_from_size_abrev(mem_size);
 	gtk_label_set(GTK_LABEL(tab->label_image_size), buf);
@@ -247,6 +260,7 @@ static void info_tab_general_sync(InfoData *id, gpointer data)
 	g_free(buf);
 
 	gtk_label_set(GTK_LABEL(tab->label_dimensions), "");
+	gtk_label_set(GTK_LABEL(tab->label_transparent), "");
 	gtk_label_set(GTK_LABEL(tab->label_image_size), "");
 
 	gtk_label_set(GTK_LABEL(tab->label_compression), "");
@@ -277,9 +291,10 @@ static TabData *info_tab_general_new(InfoData *id)
 	tab->label_file_size = table_add_line(table, 0, 1, _("File size:"), NULL);
 
 	tab->label_dimensions = table_add_line(table, 0, 2, _("Dimensions:"), NULL);
-	tab->label_image_size = table_add_line(table, 0, 3, _("Image size:"), NULL);
+	tab->label_transparent = table_add_line(table, 0, 3, _("Transparent:"), NULL);
+	tab->label_image_size = table_add_line(table, 0, 4, _("Image size:"), NULL);
 
-	tab->label_compression = table_add_line(table, 0, 4, _("Compress ratio:"), NULL);
+	tab->label_compression = table_add_line(table, 0, 5, _("Compress ratio:"), NULL);
 
 	label = gtk_label_new(_("General"));
 	gtk_notebook_append_page(GTK_NOTEBOOK(id->notebook), table, label);
@@ -373,6 +388,63 @@ static void info_window_sync(InfoData *id, const gchar *path)
 	image_change_path(id->image, path, 0.0);
 }
 
+/*
+ *-------------------------------------------------------------------
+ * drag n drop (dropping not supported!)
+ *-------------------------------------------------------------------
+ */
+
+static void info_window_dnd_data_set(GtkWidget *widget, GdkDragContext *context,
+				     GtkSelectionData *selection_data, guint info,
+				     guint time, gpointer data)
+{
+	InfoData *id = data;
+	const gchar *path;
+
+	path = image_get_path(id->image);
+	if (path)
+		{
+		gchar *text = NULL;
+		switch (info)
+			{
+			case TARGET_URI_LIST:
+				text = g_strconcat("file:", path, "\r\n", NULL);
+				break;
+			case TARGET_TEXT_PLAIN:
+				text = g_strdup(path);
+				break;
+			}
+		if (text)
+			{
+			gtk_selection_data_set(selection_data, selection_data->target,
+					       8, text, strlen(text));
+			g_free(text);
+			}
+		}
+}
+
+static void info_window_dnd_init(InfoData *id)
+{
+	ImageWindow *imd;
+
+	imd = id->image;
+
+	gtk_drag_source_set(imd->image, GDK_BUTTON2_MASK,
+			    dnd_file_drag_types, dnd_file_drag_types_count,
+			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
+	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_get",
+			   GTK_SIGNAL_FUNC(info_window_dnd_data_set), id);
+
+#if 0
+	gtk_drag_dest_set(imd->image,
+			  GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			  dnd_file_drop_types, dnd_file_drop_types_count,
+			  GDK_ACTION_COPY);
+	gtk_signal_connect(GTK_OBJECT(imd->image), "drag_data_received",
+			   GTK_SIGNAL_FUNC(info_window_dnd_data_get), id);
+#endif
+}
+
 /*
  *-------------------------------------------------------------------
  * base window
@@ -561,6 +633,8 @@ void info_window_new(const gchar *path, GList *list)
 
 	/* finish */
 
+	info_window_dnd_init(id);
+
 	gtk_widget_show(id->window);
 }
 
