@@ -43,6 +43,9 @@
 GList *layout_window_list = NULL;
 
 
+static void layout_list_scroll_to_subpart(LayoutWindow *lw, const gchar *needle);
+
+
 /*
  *-----------------------------------------------------------------------------
  * misc
@@ -119,10 +122,7 @@ static void layout_path_entry_tab_cb(const gchar *path, gpointer data)
 		}
 	else if (lw->path && strcmp(lw->path, base) == 0)
 		{
-		printf("FIXME: scroll to best match\n");
-#if 0
-		layout_list_best_match(lw, buf);
-#endif
+		layout_list_scroll_to_subpart(lw, filename_from_path(buf));
 		}
 
 	g_free(base);
@@ -533,6 +533,15 @@ static void layout_list_sync_thumb(LayoutWindow *lw)
 	if (lw->vfl) vflist_thumb_set(lw->vfl, lw->thumbs_enabled);
 }
 
+static void layout_list_scroll_to_subpart(LayoutWindow *lw, const gchar *needle)
+{
+	if (!lw) return;
+#if 0
+	if (lw->vfl) vflist_scroll_to_subpart(lw->vfl, needle);
+	if (lw->vfi) vficon_scroll_to_subpart(lw->vfi, needle);
+#endif
+}
+
 GList *layout_list(LayoutWindow *lw)
 {
 	if (!layout_valid(&lw)) return NULL;
@@ -589,7 +598,6 @@ static void layout_list_sync_sort(LayoutWindow *lw)
 	if (lw->vfi) vficon_sort_set(lw->vfi, lw->sort_method, lw->sort_ascend);
 }
 
-
 GList *layout_selection_list(LayoutWindow *lw)
 {
 	if (!layout_valid(&lw)) return NULL;
@@ -652,6 +660,8 @@ static void layout_sync_path(LayoutWindow *lw)
 {
 	if (!lw->path) return;
 
+	lw->last_time = filetime(lw->path);
+
 	gtk_entry_set_text(GTK_ENTRY(lw->path_entry), lw->path);
 	if (lw->vdl) vdlist_set_path(lw->vdl, lw->path);
 	if (lw->vdt) vdtree_set_path(lw->vdt, lw->path);
@@ -716,22 +726,66 @@ gint layout_set_path(LayoutWindow *lw, const gchar *path)
 			layout_image_set_path(lw, path);
 			}
 		}
+	else if (!lazy_image_sync)
+		{
+		layout_image_set_index(lw, 0);
+		}
 
 	return TRUE;
 }
 
-void layout_refresh(LayoutWindow *lw)
+static void layout_refresh_lists(LayoutWindow *lw)
 {
-	if (!layout_valid(&lw)) return;
+	if (lw->path) lw->last_time = filetime(lw->path);
 
 	if (lw->vdl) vdlist_refresh(lw->vdl);
 	if (lw->vdt) vdtree_refresh(lw->vdt);
 
 	if (lw->vfl) vflist_refresh(lw->vfl);
 	if (lw->vfi) vficon_refresh(lw->vfi);
+}
+
+static void layout_refresh_by_time(LayoutWindow *lw)
+{
+	layout_refresh_lists(lw);
+
+	if (lw->image && filetime(layout_image_get_path(lw)) >= lw->last_time)
+		{
+		layout_image_refresh(lw);
+		}
+}
+
+void layout_refresh(LayoutWindow *lw)
+{
+	if (!layout_valid(&lw)) return;
+
+	if (debug) printf("layout refresh\n");
+
+	layout_refresh_lists(lw);
+
 	if (lw->image) layout_image_refresh(lw);
 }
 
+static gint layout_check_for_update_cb(gpointer data)
+{
+	LayoutWindow *lw = data;
+
+	if (lw->path)
+		{
+		time_t new_time;
+
+		new_time = filetime(lw->path);
+
+		if (new_time > 0 && new_time > lw->last_time)
+			{
+			if (debug) printf("layout path time changed, refreshing...\n");
+			layout_refresh_by_time(lw);
+			}
+		}
+
+	return TRUE;
+}
+
 void layout_thumb_set(LayoutWindow *lw, gint enable)
 {
 	if (!layout_valid(&lw)) return;
@@ -786,7 +840,7 @@ gint layout_geometry_get_dividers(LayoutWindow *lw, gint *h, gint *v)
 {
 	if (!layout_valid(&lw)) return FALSE;
 
-	if (lw->h_pane)
+	if (lw->h_pane && GTK_PANED(lw->h_pane)->child1->allocation.x >= 0)
 		{
 		*h = GTK_PANED(lw->h_pane)->child1->allocation.width;
 		}
@@ -795,7 +849,7 @@ gint layout_geometry_get_dividers(LayoutWindow *lw, gint *h, gint *v)
 		*h = lw->div_h;
 		}
 
-	if (lw->v_pane)
+	if (lw->v_pane && GTK_PANED(lw->v_pane)->child1->allocation.x >= 0)
 		{
 		*v = GTK_PANED(lw->v_pane)->child1->allocation.height;
 		}
@@ -1236,15 +1290,9 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	gtk_widget_ref(old_image->widget);
 	gtk_container_remove(GTK_CONTAINER(old_image->widget->parent), old_image->widget);
 
-	gtk_container_remove(GTK_CONTAINER(lw->main_box), lw->group_box);
-
-	if (lw->menu_fact) gtk_object_unref(GTK_OBJECT(lw->menu_fact));
-
-	lw->group_box = NULL;
 	lw->h_pane = NULL;
 	lw->v_pane = NULL;
 
-	lw->menu_fact = NULL;
 	lw->toolbar = NULL;
 	lw->thumb_button = NULL;
 	lw->path_entry = NULL;
@@ -1263,6 +1311,12 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 	lw->info_details = NULL;
 	lw->info_zoom = NULL;
 
+	gtk_container_remove(GTK_CONTAINER(lw->main_box), lw->group_box);
+	lw->group_box = NULL;
+
+	if (lw->menu_fact) gtk_object_unref(GTK_OBJECT(lw->menu_fact));
+	lw->menu_fact = NULL;
+
 	/* re-fill */
 
 	layout_grid_setup(lw);
@@ -1274,8 +1328,16 @@ void layout_style_set(LayoutWindow *lw, gint style, const gchar *order)
 
 	/* sync */
 
-	layout_set_path(lw, path);
+	if (image_get_path(old_image))
+		{
+		layout_set_path(lw, image_get_path(old_image));
+		}
+	else
+		{
+		layout_set_path(lw, path);
+		}
 	image_change_from_image(lw->image, old_image);
+	image_top_window_set_sync(lw->image, (lw->tools_float || lw->tools_hidden));
 
 	/* clean up */
 
@@ -1429,7 +1491,13 @@ void layout_free(LayoutWindow *lw)
 
 	layout_window_list = g_list_remove(layout_window_list, lw);
 
+	if (lw->last_time_id != -1)
+		{
+		gtk_timeout_remove(lw->last_time_id);
+		}
+
 	if (lw->menu_fact) gtk_object_unref(GTK_OBJECT(lw->menu_fact));
+	if (lw->tooltips) gtk_object_unref(GTK_OBJECT(lw->tooltips));
 	gtk_widget_destroy(lw->window);
 
 	g_free(lw->path);
@@ -1437,7 +1505,7 @@ void layout_free(LayoutWindow *lw)
 	g_free(lw);
 }
 
-static gint layout_delete_cb(GtkWidget *widget, gpointer data)
+static gint layout_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
 {
 	LayoutWindow *lw = data;
 
@@ -1523,6 +1591,7 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 	gtk_widget_show(lw->main_box);
 
 	layout_grid_setup(lw);
+	image_top_window_set_sync(lw->image, (lw->tools_float || lw->tools_hidden));
 
 	layout_util_sync(lw);
 	layout_status_update_all(lw);
@@ -1540,6 +1609,10 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
 		gdk_pixbuf_unref(pixbuf);
 		}
 
+	/* set up the time stat timeout */
+	lw->last_time = 0;
+	lw->last_time_id = gtk_timeout_add(5000, layout_check_for_update_cb, lw);
+
 	gtk_widget_show(lw->window);
 	layout_tools_hide(lw, lw->tools_hidden);
 
@@ -1554,12 +1627,23 @@ LayoutWindow *layout_new(const gchar *path, gint popped, gint hidden)
  *-----------------------------------------------------------------------------
  */
 
+static void layout_real_time_update(LayoutWindow *lw)
+{
+	/* this resets the last time stamp of path so that a refresh does not occur
+	 * from an internal file operation.
+	 */
+
+	if (lw->path) lw->last_time = filetime(lw->path);
+}
+
 static void layout_real_renamed(LayoutWindow *lw, const gchar *source, const gchar *dest)
 {
 	if (lw->image) layout_image_maint_renamed(lw, source, dest);
 
 	if (lw->vfl) vflist_maint_renamed(lw->vfl, source, dest);
 	if (lw->vfi) vficon_maint_renamed(lw->vfi, source, dest);
+
+	layout_real_time_update(lw);
 }
 
 static void layout_real_removed(LayoutWindow *lw, const gchar *path, GList *ignore_list)
@@ -1568,6 +1652,8 @@ static void layout_real_removed(LayoutWindow *lw, const gchar *path, GList *igno
 
 	if (lw->vfl) vflist_maint_removed(lw->vfl, path, ignore_list);
 	if (lw->vfi) vficon_maint_removed(lw->vfi, path, ignore_list);
+
+	layout_real_time_update(lw);
 }
 
 static void layout_real_moved(LayoutWindow *lw, const gchar *source, const gchar *dest, GList *ignore_list)
@@ -1576,6 +1662,8 @@ static void layout_real_moved(LayoutWindow *lw, const gchar *source, const gchar
 
 	if (lw->vfl) vflist_maint_moved(lw->vfl, source, dest, ignore_list);
 	if (lw->vfi) vficon_maint_moved(lw->vfi, source, dest, ignore_list);
+
+	layout_real_time_update(lw);
 }
 
 void layout_maint_renamed(const gchar *source, const gchar *dest)
@@ -1613,4 +1701,3 @@ void layout_maint_moved(const gchar *source, const gchar *dest, GList *ignore_li
 		layout_real_moved(lw, source, dest, ignore_list);
 		}
 }
-
