@@ -285,7 +285,7 @@ static void layout_config_row_move(GtkWidget *clist, gint source_row, gint dest_
 GtkWidget *layout_config_new(void)
 {
 	LayoutConfig *lc;
-	GtkWidget *vbox;
+	GtkWidget *hbox;
 	GtkWidget *group = NULL;
 	GtkWidget *scrolled;
 	GtkWidget *label;
@@ -294,28 +294,26 @@ GtkWidget *layout_config_new(void)
 
 	lc = g_new0(LayoutConfig, 1);
 
-	lc->box = gtk_hbox_new(TRUE, 5);
+	lc->box = gtk_vbox_new(FALSE, 5);
 	gtk_object_set_data(GTK_OBJECT(lc->box), "layout_config", lc);
+	gtk_container_border_width(GTK_CONTAINER(lc->box), 5);
 
 	gtk_signal_connect(GTK_OBJECT(lc->box), "destroy",
 			   GTK_SIGNAL_FUNC(layout_config_destroy), lc);
 
-	vbox = gtk_vbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(lc->box), vbox, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new(TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(lc->box), hbox, FALSE, FALSE, 0);
 	for (i = 0; i < layout_config_style_count; i++)
 		{
-		group = layout_config_widget(group, vbox, i, lc);
+		group = layout_config_widget(group, hbox, i, lc);
 		lc->style_widgets = g_list_append(lc->style_widgets, group);
 		}
-	gtk_widget_show(vbox);
-
-	vbox = gtk_vbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(lc->box), vbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 				       GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(lc->box), scrolled, FALSE, FALSE, 0);
 	gtk_widget_show(scrolled);
 
 	lc->clist = gtk_clist_new(2);
@@ -343,11 +341,9 @@ GtkWidget *layout_config_new(void)
 	gtk_widget_show(lc->clist);
 
 	label = gtk_label_new(_("(drag to change order)"));
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(lc->box), label, FALSE, FALSE, 0);
 	gtk_widget_show(label);
 	
-	gtk_widget_show(vbox);
-
 	return lc->box;
 }
 
