@@ -268,6 +268,9 @@ void layout_image_full_screen_start(LayoutWindow *lw)
 
 	gtk_signal_connect(GTK_OBJECT(lw->full_screen->window), "key_press_event",
 			   GTK_SIGNAL_FUNC(layout_image_full_screen_key_press_cb), lw);
+
+	gtk_widget_set_sensitive(lw->window, FALSE);
+	if (lw->tools) gtk_widget_set_sensitive(lw->tools, FALSE);
 }
 
 void layout_image_full_screen_stop(LayoutWindow *lw)
@@ -276,6 +279,9 @@ void layout_image_full_screen_stop(LayoutWindow *lw)
 	if (!lw->full_screen) return;
 
 	fullscreen_stop(lw->full_screen);
+
+	gtk_widget_set_sensitive(lw->window, TRUE);
+	if (lw->tools) gtk_widget_set_sensitive(lw->tools, TRUE);
 }
 
 void layout_image_full_screen_toggle(LayoutWindow *lw)
@@ -1162,7 +1168,7 @@ static void layout_image_button4_cb(ImageWindow *imd, GdkEventButton *bevent, gp
 
 	if (bevent->state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust(imd, get_zoom_increment());
+		image_zoom_adjust_at_point(imd, get_zoom_increment(), bevent->x, bevent->y);
 		}
 	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
@@ -1180,7 +1186,7 @@ static void layout_image_button5_cb(ImageWindow *imd, GdkEventButton *bevent, gp
 
 	if (bevent->state & GDK_CONTROL_MASK)
 		{
-		image_zoom_adjust(imd, -get_zoom_increment());
+		image_zoom_adjust_at_point(imd, -get_zoom_increment(), bevent->x, bevent->y);
 		}
 	else if ( (bevent->state & GDK_SHIFT_MASK) != (mousewheel_scrolls))
 		{
@@ -1254,6 +1260,24 @@ void layout_image_maint_removed(LayoutWindow *lw, const gchar *path)
 	img_path = layout_image_get_path(lw);
 	if (img_path && strcmp(img_path, path) == 0)
 		{
+		CollectionData *cd;
+		CollectInfo *info;
+
+		cd = image_get_collection(lw->image, &info);
+		if (cd && info)
+			{
+			CollectInfo *new;
+
+			new = collection_next_by_info(cd, info);
+			if (!new) new = collection_prev_by_info(cd, info);
+
+			if (new)
+				{
+				layout_image_set_collection(lw, cd, new);
+				return;
+				}
+			}
+
 		layout_image_set_path(lw, NULL);
 		}
 }
