@@ -166,7 +166,7 @@ static gint layout_key_press_cb(GtkWidget *widget, GdkEventKey *event, gpointer
 				}
 			break;
 		case GDK_Escape:
-			printf("FIXME:interrupting thumbs no longer allowed\n");
+			/* FIXME:interrupting thumbs no longer allowed */
 #if 0
 			interrupt_thumbs();
 #endif
@@ -211,7 +211,17 @@ void layout_keyboard_init(LayoutWindow *lw)
  *-----------------------------------------------------------------------------
  * menu callbacks
  *-----------------------------------------------------------------------------
- */ 
+ */
+
+static void layout_menu_new_window_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+	LayoutWindow *nw;
+
+	nw = layout_new(NULL, FALSE, FALSE);
+	layout_sort_set(nw, file_sort_method, file_sort_ascending);
+	layout_set_path(nw, layout_get_path(lw));
+}
 
 static void layout_menu_new_cb(gpointer data, guint action, GtkWidget *widget)
 {
@@ -263,6 +273,13 @@ static void layout_menu_delete_cb(gpointer data, guint action, GtkWidget *widget
 	file_util_delete(NULL, layout_selection_list(lw));
 }
 
+static void layout_menu_close_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	LayoutWindow *lw = data;
+
+	layout_close(lw);
+}
+
 static void layout_menu_exit_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	exit_gqview();
@@ -611,6 +628,7 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 {
 	{ N_("/_File"),				NULL,		NULL,				0, "<Branch>" },
 	{ N_("/File/tear1"),			NULL,		NULL,				0, "<Tearoff>" },
+	{ N_("/File/New _window"),		NULL, 		layout_menu_new_window_cb,	0, NULL },
 	{ N_("/File/_New collection"),		"C",		layout_menu_new_cb,		0, NULL },
 	{ N_("/File/_Open collection..."),	"O",		layout_menu_open_cb,		0, NULL },
 	{ N_("/File/Open _recent"),		NULL,		NULL,				0, NULL },
@@ -624,6 +642,7 @@ static GtkItemFactoryEntry gqview_menu_items[] =
 	{ N_("/File/_Rename..."),		"<control>R",	layout_menu_rename_cb,		0, NULL },
 	{ N_("/File/_Delete..."),		"<control>D",	layout_menu_delete_cb,		0, NULL },
 	{ N_("/File/sep4"),			NULL,		NULL,				0, "<Separator>" },
+	{ N_("/File/C_lose window"),		"<control>W",	layout_menu_close_cb,		0, NULL },
 	{ N_("/File/E_xit"),			"<control>Q",	layout_menu_exit_cb,		0, NULL },
 
 	{ N_("/_Edit"),				NULL,		NULL,				0, "<Branch>" },
@@ -818,23 +837,23 @@ GtkWidget *layout_button_bar(LayoutWindow *lw)
 	box = gtk_hbox_new(FALSE, 0);
 
 	lw->thumb_button = layout_button(box, (gchar **)icon_thumb_xpm, TRUE,
-		      tooltips, _("Show thumbnails"), layout_button_thumb_cb, lw);
+		      tooltips, _("Show thumbnails"), GTK_SIGNAL_FUNC(layout_button_thumb_cb), lw);
 	layout_button(box, (gchar **)icon_home_xpm, FALSE,
-		      tooltips, _("Change to home directory"), layout_button_home_cb, lw);
+		      tooltips, _("Change to home directory"), GTK_SIGNAL_FUNC(layout_button_home_cb), lw);
 	layout_button(box, (gchar **)icon_reload_xpm, FALSE,
-		      tooltips, _("Refresh file list"), layout_button_refresh_cb, lw);
+		      tooltips, _("Refresh file list"), GTK_SIGNAL_FUNC(layout_button_refresh_cb), lw);
 	layout_button(box, (gchar **)icon_zoom_in_xpm, FALSE,
-		      tooltips, _("Zoom in"), layout_button_zoom_in_cb, lw);
+		      tooltips, _("Zoom in"), GTK_SIGNAL_FUNC(layout_button_zoom_in_cb), lw);
 	layout_button(box, (gchar **)icon_zoom_out_xpm, FALSE,
-		      tooltips, _("Zoom out"), layout_button_zoom_out_cb, lw);
+		      tooltips, _("Zoom out"), GTK_SIGNAL_FUNC(layout_button_zoom_out_cb), lw);
 	layout_button(box, (gchar **)icon_zoom_fit_xpm, FALSE,
-		      tooltips, _("Fit image to window"), layout_button_zoom_fit_cb, lw);
+		      tooltips, _("Fit image to window"), GTK_SIGNAL_FUNC(layout_button_zoom_fit_cb), lw);
 	layout_button(box, (gchar **)icon_zoom_norm_xpm, FALSE,
-		      tooltips, _("Set zoom 1:1"), layout_button_zoom_1_1_cb, lw);
+		      tooltips, _("Set zoom 1:1"), GTK_SIGNAL_FUNC(layout_button_zoom_1_1_cb), lw);
 	layout_button(box, (gchar **)icon_config_xpm, FALSE,
-		      tooltips, _("Configure options"), layout_button_config_cb, lw);
+		      tooltips, _("Configure options"), GTK_SIGNAL_FUNC(layout_button_config_cb), lw);
 	layout_button(box, (gchar **)icon_float_xpm, FALSE,
-		      tooltips, _("Float Controls"), layout_button_float_cb, lw);
+		      tooltips, _("Float Controls"), GTK_SIGNAL_FUNC(layout_button_float_cb), lw);
 
 	return box;
 }
