@@ -138,36 +138,22 @@ void help_window_show(const gchar *key)
  *-----------------------------------------------------------------------------
  * external editor start routines (public)
  *-----------------------------------------------------------------------------
- */ 
+ */
 
-void start_editor_from_file(gint n, const gchar *path)
+static void editor_line_break(gchar *command, const gchar **end)
 {
-	gchar current_path[512];
-	gchar *success;
-	gchar *cmd;
+	gchar *found;
 
-	if (!path) return;
+	found = strstr(command, "%f");
 
-	cmd = g_strdup_printf("%s \"%s\" &", editor_command[n], path);
-	if (debug) printf(_("GQview running: %s\n"),cmd);
-
-	success = getcwd(current_path, sizeof(current_path));
-
-	/* set working dir for the called application */
-	if (success)
+	if (found)
 		{
-		gchar *base;
-		base = remove_level_from_path(path);
-		if (chdir(base) != 0) success = NULL;
-		g_free(base);
+		*found = '\0';
+		*end = found + 2;
 		}
-
-	system(cmd);
-	g_free(cmd);
-
-	if (success)
+	else
 		{
-		chdir(success);
+		*end = "";
 		}
 }
 
@@ -178,21 +164,32 @@ void start_editor_from_path_list(gint n, GList *list)
 	gchar *cmd;
 	gchar *buf;
 	GList *work;
+	gchar *front;
+	const gchar *end;
 
-	if (!list) return;
+	if (!list ||
+	    !editor_command[n] ||
+	    strlen(editor_command[n]) == 0) return;
+
+	front = g_strdup(editor_command[n]);
+	editor_line_break(front, &end);
+
+	cmd = g_strconcat(front, (strlen(end) > 0) ? "" : " ", NULL);
 
 	work = list;
-	cmd = g_strconcat(editor_command[n], " ", NULL);
 	while (work)
 		{
 		buf = cmd;
-		cmd = g_strconcat(buf, "\"", (gchar *)work->data, "\" ", NULL);
+		cmd = g_strconcat(buf, "\"", (gchar *)work->data, "\"", (work->next) ? " " : "", NULL);
 		g_free(buf);
 		work = work->next;
 		}
 	buf = cmd;
-	cmd = g_strconcat(buf, "&", NULL);
+	cmd = g_strconcat(buf, end, (strlen(end) > 0) ? " " : "", "&", NULL);
 	g_free(buf);
+
+	g_free(front);
+
 	if (debug) printf(_("GQview running: %s\n"), cmd);
 
         success = getcwd(current_path, sizeof(current_path));
@@ -215,6 +212,17 @@ void start_editor_from_path_list(gint n, GList *list)
 		}
 }
 
+void start_editor_from_file(gint n, const gchar *path)
+{
+	GList *list;
+
+	if (!path) return;
+
+	list = g_list_append(NULL, (gchar *)path);
+	start_editor_from_path_list(n, list);
+	g_list_free(list);
+}
+
 /*
  *-----------------------------------------------------------------------------
  * keyboard functions
@@ -554,7 +562,9 @@ int main (int argc, char *argv[])
 	check_for_home_path(GQVIEW_RC_DIR_THUMBS);
 	check_for_home_path(GQVIEW_RC_DIR_COLLECTIONS);
 
+#if 0
 	printf("GQview %s\nThis is a beta release, in active development.\nThings may break from version to version.\nWant stability? Use a 1.0 version. YOU WERE WARNED!\n", VERSION);
+#endif
 
 	gtk_init (&argc, &argv);
 
@@ -595,7 +605,6 @@ int main (int argc, char *argv[])
 
 	lw = layout_new(NULL, tools_float, tools_hidden);
 	layout_sort_set(lw, file_sort_method, file_sort_ascending);
-	layout_set_path(lw, path);
 
 	if (cmd_list)
 		{
@@ -628,6 +637,7 @@ int main (int argc, char *argv[])
 			work = work->next;
 			}
 
+		layout_set_path(lw, path);
 		if (cd->list) layout_image_set_collection(lw, cd, cd->list->data);
 
 		/* mem leak, we never unref this collection when !startup_command_line_collection
@@ -645,7 +655,7 @@ int main (int argc, char *argv[])
 		}
 	else
 		{
-		layout_image_set_with_ahead(lw, NULL, NULL);
+		layout_set_path(lw, path);
 		}
 
 	if (collection_list)
