@@ -165,7 +165,7 @@ static void tile_cache_max_cb(GtkObject *adj, gpointer data)
 
 static void zoom_increment_cb(GtkObject *adj, gpointer data)
 {
-	zoom_increment_c = (gint)(GTK_ADJUSTMENT(adj)->value * 10.0);
+	zoom_increment_c = (gint)(GTK_ADJUSTMENT(adj)->value * 10.0 + 0.01);
 }
 
 static void cache_clear_ok_cb(GenericDialog *gd, gpointer data)
@@ -192,7 +192,8 @@ static void cache_clear_cb(GtkWidget *widget, gpointer data)
 
 static void config_window_apply(void)
 {
-	gchar *buf;
+	const gchar *buf;
+	gchar *l_conf;
 	gint new_style;
 	gint i;
 	gint refresh = FALSE;
@@ -299,10 +300,10 @@ static void config_window_apply(void)
 
 	tree_descend_subdirs = tree_descend_subdirs_c;
 
-	buf = layout_config_get(layout_widget, &new_style);
+	l_conf = layout_config_get(layout_widget, &new_style);
 
 	if (new_style != layout_style ||
-	    (buf == NULL) != (layout_order == NULL) ||
+	    (l_conf == NULL) != (layout_order == NULL) ||
 	    !layout_order ||
 	    strcmp(buf, layout_order) != 0)
 		{
@@ -310,13 +311,16 @@ static void config_window_apply(void)
 		refresh = FALSE;
 
 		g_free(layout_order);
-		layout_order = buf;
+		layout_order = l_conf;
+		l_conf = NULL;
 
 		layout_style = new_style;
 
 		layout_styles_update();
 		}
 
+	g_free(l_conf);
+
 	if (refresh)
 		{
 		filter_rebuild();
