@@ -152,6 +152,7 @@ void save_options(void)
 	fprintf(f,"\n");
 
 	write_bool_option(f, "tree_descend_folders", tree_descend_subdirs);
+	write_bool_option(f, "lazy_image_sync", lazy_image_sync);
 	fprintf(f,"\n");
 
 	write_bool_option(f, "enable_startup_path", startup_path_enable);
@@ -249,9 +250,9 @@ void save_options(void)
 	for (i=0; i<8; i++)
 		{
 		fprintf(f,"external_%d: \"", i+1);
-		if (editor_name[i]) fprintf(f, editor_name[i]);
+		if (editor_name[i]) fprintf(f, "%s", editor_name[i]);
 		fprintf(f, "\" \"");
-		if (editor_command[i]) fprintf(f, editor_command[i]);
+		if (editor_command[i]) fprintf(f, "%s", editor_command[i]);
 		fprintf(f, "\"\n");
 		}
 
@@ -344,6 +345,8 @@ void load_options(void)
 
 		tree_descend_subdirs = read_bool_option(f, option,
 			"tree_descend_folders", value, tree_descend_subdirs);
+		lazy_image_sync = read_bool_option(f, option,
+			"lazy_image_sync", value, lazy_image_sync);
 
 		startup_path_enable = read_bool_option(f, option,
 			"enable_startup_path", value, startup_path_enable);
