@@ -386,6 +386,11 @@ struct _LayoutWindow
 	gint div_h;
 	gint div_v;
 	gint div_float;
+
+	/* directory update check */
+
+	gint last_time_id;
+	time_t last_time;
 };
 
 struct _ViewDirList
