@@ -434,10 +434,11 @@ static gint dest_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer dat
 			clist_edit_set_highlight(dd->d_clist, row, TRUE);
 
 			menu = popup_menu_short_lived();
-			menu_item_add(menu, _("Rename"), dest_popup_dir_rename_cb, dd);
+			menu_item_add(menu, _("Rename"),
+				      GTK_SIGNAL_FUNC(dest_popup_dir_rename_cb), dd);
 
 			gtk_signal_connect(GTK_OBJECT(menu), "destroy",
-				GTK_SIGNAL_FUNC(dest_popup_dir_done_cb), dd);
+					   GTK_SIGNAL_FUNC(dest_popup_dir_done_cb), dd);
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
 			}
 		
@@ -457,11 +458,13 @@ static gint dest_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer dat
 			clist_edit_set_highlight(dd->f_clist, row, TRUE);
 
 			menu = popup_menu_short_lived();
-			menu_item_add(menu, _("Rename"), dest_popup_file_rename_cb, dd);
-			menu_item_add(menu, _("Delete"), dest_popup_file_delete_cb, dd);
+			menu_item_add(menu, _("Rename"),
+				      GTK_SIGNAL_FUNC(dest_popup_file_rename_cb), dd);
+			menu_item_add(menu, _("Delete"),
+				      GTK_SIGNAL_FUNC(dest_popup_file_delete_cb), dd);
 
 			gtk_signal_connect(GTK_OBJECT(menu), "destroy",
-				GTK_SIGNAL_FUNC(dest_popup_file_done_cb), dd);
+					   GTK_SIGNAL_FUNC(dest_popup_file_done_cb), dd);
 			gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, event->time);
 
 			return FALSE;
@@ -728,6 +731,7 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	GtkWidget *scrolled;
 	GtkWidget *button;
 	GtkWidget *table;
+	GtkWidget *paned;
 #ifdef PATH_SEL_USE_HEADINGS
 	gchar *d_title[] = { "Directories", NULL };
 	gchar *f_title[] = { "Files", NULL };
@@ -752,25 +756,40 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	gtk_widget_show(hbox2);
 
 	button = gtk_button_new_with_label(_("Home"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dest_home_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dest_home_cb), dd);
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("New Directory"));
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(dest_new_dir_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked",
+			   GTK_SIGNAL_FUNC(dest_new_dir_cb), dd);
 	gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
 	gtk_widget_show(button);
 
 	dd->hidden_button = gtk_check_button_new_with_label(_("Show hidden"));
-	gtk_signal_connect(GTK_OBJECT(dd->hidden_button), "clicked", GTK_SIGNAL_FUNC(dest_show_hidden_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(dd->hidden_button), "clicked",
+			   GTK_SIGNAL_FUNC(dest_show_hidden_cb), dd);
 	gtk_box_pack_end(GTK_BOX(hbox2), dd->hidden_button, FALSE, FALSE, 0);
 	gtk_widget_show(dd->hidden_button);
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_table_attach(GTK_TABLE(table), scrolled, 0, 1, 1, 2,
-			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+	if (filter)
+		{
+		paned = gtk_hpaned_new();
+		gtk_table_attach(GTK_TABLE(table), paned, 0, 3, 1, 2,
+				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+		gtk_widget_show(paned);
+		gtk_paned_add1(GTK_PANED(paned), scrolled);
+		}
+	else
+		{
+		paned = NULL;
+		gtk_table_attach(GTK_TABLE(table), scrolled, 0, 1, 1, 2,
+				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+		}
 	gtk_widget_show(scrolled);
 
 #ifdef PATH_SEL_USE_HEADINGS
@@ -780,9 +799,12 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 	dd->d_clist=gtk_clist_new(1);
 #endif
 	gtk_clist_set_column_auto_resize(GTK_CLIST(dd->d_clist), 0, TRUE);
-	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "button_press_event", (GtkSignalFunc)dest_press_cb, dd);
-        gtk_signal_connect(GTK_OBJECT(dd->d_clist), "select_row", (GtkSignalFunc)dest_select_cb, dd);
-	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy", (GtkSignalFunc)dest_free_data, dd);
+	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "button_press_event",
+			   GTK_SIGNAL_FUNC(dest_press_cb), dd);
+        gtk_signal_connect(GTK_OBJECT(dd->d_clist), "select_row",
+			   GTK_SIGNAL_FUNC(dest_select_cb), dd);
+	gtk_signal_connect(GTK_OBJECT(dd->d_clist), "destroy",
+			   GTK_SIGNAL_FUNC(dest_free_data), dd);
 	gtk_widget_set_usize(dd->d_clist, DEST_WIDTH, DEST_HEIGHT);
 	gtk_container_add (GTK_CONTAINER (scrolled), dd->d_clist);
         gtk_widget_show (dd->d_clist);
@@ -801,14 +823,22 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 		gtk_widget_show(label);
 
 		dd->filter_combo = gtk_combo_new();
+		gtk_combo_set_case_sensitive(GTK_COMBO(dd->filter_combo), TRUE);
 		gtk_box_pack_start(GTK_BOX(hbox2), dd->filter_combo, TRUE, TRUE, 0);
 		gtk_widget_show(dd->filter_combo);
 
 		scrolled = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-		gtk_table_attach(GTK_TABLE(table), scrolled, 2, 3, 1, 2,
+		if (paned)
+			{
+			gtk_paned_add2(GTK_PANED(paned), scrolled);
+			}
+		else
+			{
+			gtk_table_attach(GTK_TABLE(table), scrolled, 2, 3, 1, 2,
 				 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+			}
 		gtk_widget_show(scrolled);
 
 #ifdef PATH_SEL_USE_HEADINGS
@@ -819,8 +849,10 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 #endif
 		gtk_clist_set_column_auto_resize(GTK_CLIST(dd->f_clist), 0, TRUE);
 		gtk_widget_set_usize(dd->f_clist, DEST_WIDTH, DEST_HEIGHT);
-		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "button_press_event", (GtkSignalFunc)dest_press_cb, dd);
-		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "select_row",(GtkSignalFunc) dest_select_cb, dd);
+		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "button_press_event",
+				   GTK_SIGNAL_FUNC(dest_press_cb), dd);
+		gtk_signal_connect(GTK_OBJECT(dd->f_clist), "select_row",
+				   GTK_SIGNAL_FUNC(dest_select_cb), dd);
 		gtk_container_add(GTK_CONTAINER (scrolled), dd->f_clist);
 	        gtk_widget_show(dd->f_clist);
 
@@ -850,8 +882,13 @@ GtkWidget *path_selection_new_with_files(GtkWidget *entry, const gchar *path,
 		g_free(buf);
 		}
 
-	if (dd->filter_combo) gtk_signal_connect(GTK_OBJECT(GTK_COMBO(dd->filter_combo)->entry), "changed", dest_filter_changed_cb, dd);
-	gtk_signal_connect(GTK_OBJECT(dd->entry), "changed", dest_entry_changed_cb, dd);
+	if (dd->filter_combo)
+		{
+		gtk_signal_connect(GTK_OBJECT(GTK_COMBO(dd->filter_combo)->entry), "changed",
+				   GTK_SIGNAL_FUNC(dest_filter_changed_cb), dd);
+		}
+	gtk_signal_connect(GTK_OBJECT(dd->entry), "changed",
+			   GTK_SIGNAL_FUNC(dest_entry_changed_cb), dd);
 
 	return table;
 }
@@ -915,6 +952,3 @@ void path_selection_clear_filter(GtkWidget *entry)
 	dest_filter_clear(dd);
 }
 
-
-
-
