@@ -102,14 +102,17 @@ static gint generic_dialog_delete_cb(GtkWidget *w, GdkEventAny *event, gpointer
 static void generic_dialog_show_cb(GtkWidget *widget, gpointer data)
 {
 	GenericDialog *gd = data;
-	if (gd->cancel_cb) generic_dialog_add(gd, _("Cancel"), gd->cancel_cb, FALSE);
+	if (gd->cancel_button)
+	       	{
+		gtk_box_reorder_child(GTK_BOX(gd->hbox), gd->cancel_button, -1);
+		}
 
 	gtk_signal_disconnect_by_func(GTK_OBJECT(widget),
 				      GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
 }
 
-GtkWidget * generic_dialog_add(GenericDialog *gd, const gchar *text,
-			       void (*func_cb)(GenericDialog *, gpointer), gint is_default)
+GtkWidget *generic_dialog_add(GenericDialog *gd, const gchar *text,
+			      void (*func_cb)(GenericDialog *, gpointer), gint is_default)
 {
 	GtkWidget *button;
 	button = gtk_button_new_with_label(text);
@@ -180,8 +183,19 @@ static void generic_dialog_setup(GenericDialog *gd,
 	gtk_box_pack_start(GTK_BOX(vbox), gd->hbox, FALSE, FALSE, 0);
 	gtk_widget_show(gd->hbox);
 
+	if (gd->cancel_cb)
+	       	{
+		gd->cancel_button = generic_dialog_add(gd, _("Cancel"), gd->cancel_cb, TRUE);
+		}
+	else
+		{
+		gd->cancel_button = NULL;
+		}
+
 	gtk_signal_connect(GTK_OBJECT(gd->dialog), "show",
 			   GTK_SIGNAL_FUNC(generic_dialog_show_cb), gd);
+
+	gd->default_cb = NULL;
 }
 
 GenericDialog *generic_dialog_new(const gchar *title, const gchar *message,
