@@ -18,6 +18,7 @@
 #include "collect.h"
 #include "dupe.h"
 #include "image.h"
+#include "img-view.h"
 #include "layout.h"
 #include "ui_fileops.h"
 #include "ui_tabcomp.h"
@@ -33,6 +34,7 @@
 void file_maint_renamed(const gchar *source, const gchar *dest)
 {
 	layout_maint_renamed(source, dest);
+	view_window_maint_moved(source, dest);
 	collection_maint_renamed(source, dest);
 	dupe_maint_renamed(source, dest);
 	cache_maint_moved(source, dest);
@@ -42,6 +44,7 @@ void file_maint_renamed(const gchar *source, const gchar *dest)
 void file_maint_removed(const gchar *path, GList *ignore_list)
 {
 	layout_maint_removed(path, ignore_list);
+	view_window_maint_removed(path, ignore_list);
 	collection_maint_removed(path);
 	dupe_maint_removed(path);
 	cache_maint_removed(path);
@@ -51,6 +54,7 @@ void file_maint_removed(const gchar *path, GList *ignore_list)
 void file_maint_moved(const gchar *source, const gchar *dest, GList *ignore_list)
 {
 	layout_maint_moved(source, dest, ignore_list);
+	view_window_maint_moved(source, dest);
 	collection_maint_renamed(source, dest);
 	dupe_maint_renamed(source, dest);
 	cache_maint_moved(source, dest);
@@ -341,6 +345,15 @@ static void file_util_move_multiple_skip_cb(GenericDialog *gd, gpointer data)
 	file_util_move_multiple(fdm);
 }
 
+static void file_util_move_multiple_skip_all_cb(GenericDialog *gd, gpointer data)
+{
+	FileDataMult *fdm = data;
+
+	fdm->skip = TRUE;
+	fdm->confirm_all = TRUE;
+	file_util_move_multiple(fdm);
+}
+
 static void file_util_move_multiple_cancel_cb(GenericDialog *gd, gpointer data)
 {
 	FileDataMult *fdm = data;
@@ -431,6 +444,7 @@ static void file_util_move_multiple(FileDataMult *fdm)
 			fdm->yes_all_button = generic_dialog_add(gd, _("Yes to all"),
 								 file_util_move_multiple_all_cb, FALSE);
 			generic_dialog_add(gd, _("Skip"), file_util_move_multiple_skip_cb, FALSE);
+			generic_dialog_add(gd, _("Skip all"), file_util_move_multiple_skip_all_cb, FALSE);
 			generic_dialog_add_images(gd, fdm->dest, fdm->source);
 
 			/* rename option */
@@ -474,7 +488,7 @@ static void file_util_move_multiple(FileDataMult *fdm)
 			if (fdm->skip)
 				{
 				success = TRUE;
-				fdm->skip = FALSE;
+				if (!fdm->confirm_all) fdm->skip = FALSE;
 				}
 			else
 				{
@@ -1086,27 +1100,57 @@ void file_util_delete(const gchar *source_path, GList *source_list)
  * rename multiple files
  */
 
-static void file_util_rename_multiple(FileDialog *fd);
+typedef struct _RenameDataMult RenameDataMult;
+struct _RenameDataMult
+{
+	FileDialog *fd;
+
+	const gchar *source;
+	gint rename_auto;
+
+	GtkWidget *clist;
+	GtkWidget *button_auto;
+
+	GtkWidget *rename_box;
+	GtkWidget *rename_label;
+	GtkWidget *rename_entry;
+
+	GtkWidget *auto_box;
+	GtkWidget *auto_entry_front;
+	GtkWidget *auto_spin_start;
+	GtkWidget *auto_spin_pad;
+	GtkWidget *auto_entry_end;
+};
+
+static void file_util_rename_multiple(RenameDataMult *rd);
 
 static void file_util_rename_multiple_ok_cb(GenericDialog *gd, gpointer data)
 {
-	FileDialog *fd = data;
+	RenameDataMult *rd = data;
+	GtkWidget *dialog;
 
-	if (!GTK_WIDGET_VISIBLE(GENERIC_DIALOG(fd)->dialog)) gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
+	dialog = GENERIC_DIALOG(rd->fd)->dialog;
+	if (!GTK_WIDGET_VISIBLE(dialog)) gtk_widget_show(dialog);
 
-	fd->type = TRUE;
-	file_util_rename_multiple(fd);
+	rd->fd->type = TRUE;
+	file_util_rename_multiple(rd);
 }
 
 static void file_util_rename_multiple_cancel_cb(GenericDialog *gd, gpointer data)
 {
-	FileDialog *fd = data;
+	RenameDataMult *rd = data;
+	GtkWidget *dialog;
 
-	if (!GTK_WIDGET_VISIBLE(GENERIC_DIALOG(fd)->dialog)) gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
+	dialog = GENERIC_DIALOG(rd->fd)->dialog;
+	if (!GTK_WIDGET_VISIBLE(dialog)) gtk_widget_show(dialog);
 }
 
-static void file_util_rename_multiple(FileDialog *fd)
+static void file_util_rename_multiple(RenameDataMult *rd)
 {
+	FileDialog *fd;
+
+	fd = rd->fd;
+
 	if (isfile(fd->dest_path) && !fd->type)
 		{
 		GenericDialog *gd;
@@ -1114,7 +1158,7 @@ static void file_util_rename_multiple(FileDialog *fd)
 
 		text = g_strdup_printf(_("Overwrite file:\n%s\nby renaming:\n%s"), fd->dest_path, fd->source_path);
 		gd = file_util_gen_dlg(_("Overwrite file"), text, "GQview", "dlg_confirm", TRUE,
-					file_util_rename_multiple_cancel_cb, fd);
+					file_util_rename_multiple_cancel_cb, rd);
 		g_free(text);
 
 		generic_dialog_add(gd, _("Overwrite"), file_util_rename_multiple_ok_cb, TRUE);
@@ -1139,16 +1183,12 @@ static void file_util_rename_multiple(FileDialog *fd)
 			{
 			gint row;
 			gint n;
-			GtkWidget *clist;
-			gchar *path;
 
 			file_maint_renamed(fd->source_path, fd->dest_path);
 
-			clist = gtk_object_get_data(GTK_OBJECT(fd->entry), "clist");
-			path = gtk_object_get_data(GTK_OBJECT(clist), "source");
-			row = gtk_clist_find_row_from_data(GTK_CLIST(clist), path);
+			row = gtk_clist_find_row_from_data(GTK_CLIST(rd->clist), (gchar *)rd->source);
 
-			n = g_list_length(GTK_CLIST(clist)->row_list);
+			n = GTK_CLIST(rd->clist)->rows;
 			if (debug) printf("r=%d n=%d\n", row, n);
 			if (n - 1 > row)
 				n = row;
@@ -1159,25 +1199,130 @@ static void file_util_rename_multiple(FileDialog *fd)
 
 			if (n >= 0)
 				{
-				gtk_object_set_data(GTK_OBJECT(clist), "source", NULL);
-				gtk_clist_remove(GTK_CLIST(clist), row);
-				gtk_clist_select_row(GTK_CLIST(clist), n, -1);
+				rd->source = NULL;
+				gtk_clist_remove(GTK_CLIST(rd->clist), row);
+				gtk_clist_select_row(GTK_CLIST(rd->clist), n, -1);
 				}
 			else
 				{
 				if (debug) printf("closed by #%d\n", n);
-				file_dialog_close(fd);
+
+				file_dialog_close(rd->fd);
+				g_free(rd);
 				}
 			}
 		}
 }
 
+static void file_util_rename_multiple_auto(RenameDataMult *rd)
+{
+	const gchar *front;
+	const gchar *end;
+	gint start_n;
+	gint padding;
+	gint n;
+	gint row;
+	gint success;
+
+	front = gtk_entry_get_text(GTK_ENTRY(rd->auto_entry_front));
+	end = gtk_entry_get_text(GTK_ENTRY(rd->auto_entry_end));
+	start_n = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(rd->auto_spin_start));
+	padding = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(rd->auto_spin_pad));
+
+	/* first check for name conflicts */
+	success = TRUE;
+	n = start_n;
+	row = 0;
+	while (row < GTK_CLIST(rd->clist)->rows && success)
+		{
+		gchar *dest;
+		gchar *base;
+		const gchar *path;
+
+		path = gtk_clist_get_row_data(GTK_CLIST(rd->clist), row);
+
+		base = remove_level_from_path(path);
+		dest = g_strdup_printf("%s/%s%0*d%s", base, front, padding, n, end);
+		if (isname(dest)) success = FALSE;
+		g_free(dest);
+		g_free(base);
+
+		n++;
+		row++;
+		}
+
+	if (!success)
+		{
+		warning_dialog(_("Auto rename"), _("Can not auto rename with the selected\nnumber set, one or more files exist that\nmatch the resulting name list.\n"));
+		return;
+		}
+
+	/* now do it for real */
+	success = TRUE;
+	n = start_n;
+	while (GTK_CLIST(rd->clist)->rows > 0 && success)
+		{
+		gchar *dest;
+		gchar *base;
+
+		base = remove_level_from_path(rd->source);
+		dest = g_strdup_printf("%s/%s%0*d%s", base, front, padding, n, end);
+		if (rename(rd->source, dest) < 0)
+			{
+			success = FALSE;
+			}
+		else
+			{
+			file_maint_renamed(rd->source, dest);
+			}
+
+		g_free(dest);
+		g_free(base);
+
+		if (success)
+			{
+			rd->source = NULL;
+			gtk_clist_remove(GTK_CLIST(rd->clist), 0);
+			if (GTK_CLIST(rd->clist)->rows > 0)
+				{
+				gtk_clist_select_row(GTK_CLIST(rd->clist), 0, -1);
+				}
+			}
+
+		n++;
+		}
+
+	if (!success)
+		{
+		gchar *buf;
+
+		n--;
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(rd->auto_spin_start), (float)n);
+
+		buf = g_strdup_printf(_("Failed to rename\n%s\nThe number was %d."), filename_from_path(rd->source), n);
+		warning_dialog(_("Auto rename"), buf);
+		g_free(buf);
+
+		return;
+		}
+
+	file_dialog_close(rd->fd);
+	g_free(rd);
+}
+
 static void file_util_rename_multiple_cb(FileDialog *fd, gpointer data)
 {
+	RenameDataMult *rd = data;
 	gchar *base;
 	const gchar *name;
 
-	name = gtk_entry_get_text(GTK_ENTRY(fd->entry));
+	if (rd->rename_auto)
+		{
+		file_util_rename_multiple_auto(rd);
+		return;
+		}
+
+	name = gtk_entry_get_text(GTK_ENTRY(rd->rename_entry));
 	base = remove_level_from_path(fd->source_path);
 
 	g_free(fd->dest_path);
@@ -1190,106 +1335,214 @@ static void file_util_rename_multiple_cb(FileDialog *fd, gpointer data)
 		}
 
 	fd->type = FALSE;
-	file_util_rename_multiple(fd);
+	file_util_rename_multiple(rd);
 }
 
 static void file_util_rename_multiple_close_cb(FileDialog *fd, gpointer data)
 {
-	file_dialog_close(fd);
+	RenameDataMult *rd = data;
+
+	file_dialog_close(rd->fd);
+	g_free(rd);
 }
 
-static void file_util_rename_multiple_select_cb(GtkWidget *clist,
-		gint row, gint column, GdkEventButton *bevent, gpointer data)
+static void file_util_rename_multiple_select_cb(GtkWidget *clist, gint row, gint column,
+						GdkEventButton *bevent, gpointer data)
 {
-	FileDialog *fd = data;
-	GtkWidget *label;
+	RenameDataMult *rd = data;
 	const gchar *name;
-	gchar *path;
+	const gchar *path;
 
-	label = gtk_object_get_data(GTK_OBJECT(GENERIC_DIALOG(fd)->dialog), "label");
 	path = gtk_clist_get_row_data(GTK_CLIST(clist), row);
 
-	g_free(fd->source_path);
-	fd->source_path = g_strdup(path);
+	g_free(rd->fd->source_path);
+	rd->fd->source_path = g_strdup(path);
 
-	gtk_object_set_data(GTK_OBJECT(clist), "source", path);
+	rd->source = path;
 
-	name = filename_from_path(fd->source_path);
-	gtk_label_set(GTK_LABEL(label), name);
-	gtk_entry_set_text(GTK_ENTRY(fd->entry), name);
+	name = filename_from_path(rd->fd->source_path);
+	gtk_label_set(GTK_LABEL(rd->rename_label), name);
+	gtk_entry_set_text(GTK_ENTRY(rd->rename_entry), name);
 
-	gtk_widget_grab_focus(fd->entry);
+	if (GTK_WIDGET_VISIBLE(rd->rename_box))
+		{
+		gtk_widget_grab_focus(rd->rename_entry);
+		}
+}
+
+static void file_util_rename_mulitple_auto_toggle(GtkWidget *widget, gpointer data)
+{
+	RenameDataMult *rd = data;
+
+	rd->rename_auto = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rd->button_auto));
+
+	if (rd->rename_auto)
+		{
+		if (GTK_WIDGET_VISIBLE(rd->rename_box)) gtk_widget_hide(rd->rename_box);
+		if (!GTK_WIDGET_VISIBLE(rd->auto_box)) gtk_widget_show(rd->auto_box);
+		}
+	else
+		{
+		if (GTK_WIDGET_VISIBLE(rd->auto_box)) gtk_widget_hide(rd->auto_box);
+		if (!GTK_WIDGET_VISIBLE(rd->rename_box)) gtk_widget_show(rd->rename_box);
+		}
+}
+
+static GtkWidget *furm_simple_label(GtkWidget *box, const gchar *text)
+{
+	GtkWidget *hbox;
+	GtkWidget *label;
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	label = gtk_label_new(text);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show(label);
+
+	return label;
+}
+
+static GtkWidget *furm_simple_vlabel(GtkWidget *box, const gchar *text)
+{
+	GtkWidget *vbox;
+	GtkWidget *label;
+
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(box), vbox, FALSE, FALSE, 0);
+	gtk_widget_show(vbox);
+
+	label = gtk_label_new(text);
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+	gtk_widget_show(label);
+
+	return vbox;
 }
 
 static void file_util_rename_multiple_do(GList *source_list)
 {
-	FileDialog *fd;
+	RenameDataMult *rd;
 	GtkWidget *scrolled;
-	GtkWidget *clist;
+	GtkWidget *hbox;
+	GtkWidget *vbox;
+	GtkWidget *box2;
 	GtkWidget *label;
+	GtkObject *adj;
 	GList *work;
 
-	fd = file_util_file_dlg(_("GQview - rename"), _("Rename multiple files:"), "GQview", "dlg_rename",
-			     file_util_rename_multiple_close_cb, NULL);
-	file_dialog_add_button(fd, _("Rename"), file_util_rename_multiple_cb, TRUE);
+	rd = g_new0(RenameDataMult, 1);
 
-	fd->source_path = g_strdup(source_list->data);
-	fd->dest_path = NULL;
+	rd->fd = file_util_file_dlg(_("GQview - rename"), _("Rename multiple files:"), "GQview", "dlg_rename",
+				    file_util_rename_multiple_close_cb, rd);
+	file_dialog_add_button(rd->fd, _("Rename"), file_util_rename_multiple_cb, TRUE);
+
+	rd->fd->source_path = g_strdup(source_list->data);
+	rd->fd->dest_path = NULL;
+
+	vbox = GENERIC_DIALOG(rd->fd)->vbox;
 
 	scrolled = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
-				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
-	gtk_box_pack_start(GTK_BOX(GENERIC_DIALOG(fd)->vbox), scrolled, TRUE, TRUE, 0);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolled),
+				       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
 	gtk_widget_show(scrolled);
 
-	clist = gtk_clist_new(1);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(clist), 0, TRUE);
-	gtk_signal_connect(GTK_OBJECT(clist), "select_row",
-			   GTK_SIGNAL_FUNC(file_util_rename_multiple_select_cb), fd);
-	gtk_widget_set_usize(clist, 250, 150);
-	gtk_container_add(GTK_CONTAINER(scrolled), clist);
-	gtk_widget_show(clist);
+	rd->clist = gtk_clist_new(1);
+	gtk_clist_set_column_auto_resize(GTK_CLIST(rd->clist), 0, TRUE);
+	gtk_clist_set_reorderable(GTK_CLIST(rd->clist), TRUE);
+	gtk_signal_connect(GTK_OBJECT(rd->clist), "select_row",
+			   GTK_SIGNAL_FUNC(file_util_rename_multiple_select_cb), rd);
+	gtk_widget_set_usize(rd->clist, 250, 150);
+	gtk_container_add(GTK_CONTAINER(scrolled), rd->clist);
+	gtk_widget_show(rd->clist);
 
-	gtk_object_set_data(GTK_OBJECT(clist), "source", source_list->data);
+	rd->source = source_list->data;
 
 	work = source_list;
-	while(work)
+	while (work)
 		{
 		gint row;
 		gchar *buf[2];
 		buf[0] = (gchar *)filename_from_path(work->data);
 		buf[1] = NULL;
-		row = gtk_clist_append(GTK_CLIST(clist), buf);
-		gtk_clist_set_row_data_full(GTK_CLIST(clist), row,
+		row = gtk_clist_append(GTK_CLIST(rd->clist), buf);
+		gtk_clist_set_row_data_full(GTK_CLIST(rd->clist), row,
 					    work->data, (GtkDestroyNotify)g_free);
 		work = work->next;
 		}
 
 	g_list_free(source_list);
 
-	label = gtk_label_new(_("Rename:"));
-	gtk_box_pack_start(GTK_BOX(GENERIC_DIALOG(fd)->vbox), label, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	rd->button_auto = gtk_check_button_new_with_label(_("Auto rename"));
+	gtk_signal_connect(GTK_OBJECT(rd->button_auto), "clicked",
+			   GTK_SIGNAL_FUNC(file_util_rename_mulitple_auto_toggle), rd);
+	gtk_box_pack_end(GTK_BOX(hbox), rd->button_auto, FALSE, FALSE, 0);
+	gtk_widget_show(rd->button_auto);
+
+	rd->rename_box = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), rd->rename_box, FALSE, FALSE, 0);
+	gtk_widget_show(rd->rename_box);
+
+	furm_simple_label(rd->rename_box, _("Rename:"));
+	rd->rename_label = furm_simple_label(rd->rename_box, filename_from_path(rd->fd->source_path));
+	furm_simple_label(rd->rename_box, _("to:"));
+
+	rd->rename_entry = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(rd->rename_entry), filename_from_path(rd->fd->source_path));
+	gtk_box_pack_start(GTK_BOX(rd->rename_box), rd->rename_entry, FALSE, FALSE, 0);
+	generic_dialog_attach_default(GENERIC_DIALOG(rd->fd), rd->rename_entry);
+	gtk_widget_grab_focus(rd->rename_entry);
+	gtk_widget_show(rd->rename_entry);
+
+	rd->auto_box = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), rd->auto_box, FALSE, FALSE, 0);
+	/* do not show it here */
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(rd->auto_box), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
+	box2 = furm_simple_vlabel(hbox, _("Begin text"));
+
+	rd->auto_entry_front = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(rd->auto_entry_front), "");
+	gtk_box_pack_start(GTK_BOX(box2), rd->auto_entry_front, TRUE, TRUE, 0);
+	gtk_widget_show(rd->auto_entry_front);
+	
+	box2 = furm_simple_vlabel(hbox, _("Start #"));
+
+	adj = gtk_adjustment_new(0.0, 1.0, 1000000.0, 1.0, 1.0, 10.0);
+	rd->auto_spin_start = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
+	gtk_widget_set_usize(rd->auto_spin_start, 70, -1);
+	gtk_box_pack_start(GTK_BOX(box2), rd->auto_spin_start, FALSE, FALSE, 0);
+	gtk_widget_show(rd->auto_spin_start);
+
+	box2 = furm_simple_vlabel(hbox, _("End text"));
+
+	rd->auto_entry_end = gtk_entry_new();
+	gtk_entry_set_text(GTK_ENTRY(rd->auto_entry_end), "");
+	gtk_box_pack_start(GTK_BOX(box2), rd->auto_entry_end, TRUE, TRUE, 0);
+	gtk_widget_show(rd->auto_entry_end);
+
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(rd->auto_box), hbox, FALSE, FALSE, 5);
+	gtk_widget_show(hbox);
+
+	label = gtk_label_new(_("Padding:"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 	gtk_widget_show(label);
 
-	label = gtk_label_new(filename_from_path(fd->source_path));
-	gtk_box_pack_start(GTK_BOX(GENERIC_DIALOG(fd)->vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-	gtk_object_set_data(GTK_OBJECT(GENERIC_DIALOG(fd)->dialog), "label", label);
+	adj = gtk_adjustment_new(1.0, 1.0, 8.0, 1.0, 1.0, 1.0);
+	rd->auto_spin_pad = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), rd->auto_spin_pad, FALSE, FALSE, 0);
+	gtk_widget_show(rd->auto_spin_pad);
 
-	label = gtk_label_new(_("to:"));
-	gtk_box_pack_start(GTK_BOX(GENERIC_DIALOG(fd)->vbox), label, FALSE, FALSE, 0);
-	gtk_widget_show(label);
-
-	fd->entry = gtk_entry_new();
-	gtk_entry_set_text(GTK_ENTRY(fd->entry), filename_from_path(fd->source_path));
-	gtk_box_pack_start(GTK_BOX(GENERIC_DIALOG(fd)->vbox), fd->entry, FALSE, FALSE, 0);
-	generic_dialog_attach_default(GENERIC_DIALOG(fd), fd->entry);
-	gtk_widget_grab_focus(fd->entry);
-	gtk_widget_show(fd->entry);
-
-	gtk_object_set_data(GTK_OBJECT(fd->entry), "clist", clist);
-
-	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
+	gtk_widget_show(GENERIC_DIALOG(rd->fd)->dialog);
 }
 
 /*
