@@ -537,18 +537,18 @@ static void vdlist_dnd_init(ViewDirList *vdl)
 			    dnd_file_drag_types, dnd_file_drag_types_count,
 			    GDK_ACTION_COPY | GDK_ACTION_MOVE | GDK_ACTION_LINK);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "drag_data_get",
-			   vdlist_dnd_get, vdl);
+			   GTK_SIGNAL_FUNC(vdlist_dnd_get), vdl);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "drag_begin",
-			   vdlist_dnd_begin, vdl);
+			   GTK_SIGNAL_FUNC(vdlist_dnd_begin), vdl);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "drag_end",
-			   vdlist_dnd_end, vdl);
+			   GTK_SIGNAL_FUNC(vdlist_dnd_end), vdl);
 
 	gtk_drag_dest_set(vdl->clist,
 			GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
 			vdlist_dnd_drop_types, vdlist_dnd_drop_types_count,
 			GDK_ACTION_MOVE | GDK_ACTION_COPY);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "drag_data_received",
-			vdlist_dnd_drop_receive, vdl);
+			   GTK_SIGNAL_FUNC(vdlist_dnd_drop_receive), vdl);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "drag_motion",
 			GTK_SIGNAL_FUNC(vdlist_dnd_drop_motion), vdl);
 	gtk_signal_connect(GTK_OBJECT(vdl->clist), "drag_leave",
