@@ -1111,6 +1111,9 @@ gint vdtree_set_path(ViewDirTree *vdt, const gchar *path)
 	gtk_ctree_select(GTK_CTREE(vdt->ctree), node);
 	vdtree_make_visible(vdt, node);
 
+	/* expand selected to display subdirs */
+	gtk_ctree_expand(GTK_CTREE(vdt->ctree), node);
+
 	return TRUE;
 }
 
