@@ -266,7 +266,7 @@ static GtkWidget *vficon_pop_menu(ViewFileIcon *vfi, gint active)
 
 	menu = popup_menu_short_lived();
 
-	submenu_add_edit(menu, &item, vficon_pop_menu_edit_cb, vfi);
+	submenu_add_edit(menu, &item, GTK_SIGNAL_FUNC(vficon_pop_menu_edit_cb), vfi);
 	gtk_widget_set_sensitive(item, active);
 
 	item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(vficon_pop_menu_info_cb), vfi);
@@ -423,7 +423,10 @@ static gint tip_schedule_cb(gpointer data)
 
 	if (vfi->tip_delay_id == -1) return FALSE;
 
-	tip_show(vfi);
+	if (GTK_WIDGET_SENSITIVE(gtk_widget_get_toplevel(vfi->clist)))
+		{
+		tip_show(vfi);
+		}
 
 	vfi->tip_delay_id = -1;
 	return FALSE;
@@ -1417,7 +1420,7 @@ static void vficon_populate(ViewFileIcon *vfi, gint resize, gint keep_position)
 
 	if (keep_position)
 		{
-		gtk_clist_get_selection_info(clist, 0, 0, &visible_row, 0);
+		gtk_clist_get_selection_info(clist, 0, 0, &visible_row, NULL);
 		if (clist->row_height > 0 && clist->clist_window_height > 0 && visible_row < clist->rows - 1)
 			{
 			gint icon_offset;
@@ -1474,13 +1477,13 @@ static void vficon_populate(ViewFileIcon *vfi, gint resize, gint keep_position)
 		if (col >= vfi->columns) col = 0;
 		}
 
+	gtk_clist_thaw(clist);
+
 	if (keep_position && visible_row > 0)
 		{
 		gtk_clist_moveto(clist, visible_row, -1, visible_offset, 0.0);
 		}
 
-	gtk_clist_thaw(clist);
-
 	vfi->rows = row + 1;
 
 	vficon_send_update(vfi);
@@ -2216,7 +2219,8 @@ void vficon_maint_removed(ViewFileIcon *vfi, const gchar *path, GList *ignore_li
 	fd = g_list_nth_data(vfi->list, row);
 	if (!fd) return;
 
-	if (vficon_index_is_selected(vfi, row))
+	if (vficon_index_is_selected(vfi, row) &&
+	    layout_image_get_collection(vfi->layout, NULL) == NULL)
 		{
 		vficon_unselect(vfi, fd);
 
