@@ -258,7 +258,7 @@ static GtkWidget *vflist_pop_menu(ViewFileList *vfl, gint active)
 
 	menu = popup_menu_short_lived();
 
-	submenu_add_edit(menu, &item, vflist_pop_menu_edit_cb, vfl);
+	submenu_add_edit(menu, &item, GTK_SIGNAL_FUNC(vflist_pop_menu_edit_cb), vfl);
 	gtk_widget_set_sensitive(item, active);
 
 	item = menu_item_add(menu, _("Properties"), GTK_SIGNAL_FUNC(vflist_pop_menu_info_cb), vfl);
@@ -847,7 +847,7 @@ gint vflist_selection_count(ViewFileList *vfl, gint *bytes)
 			work = work->next;
 			
 			fd = vflist_index_get_data(vfl, row);
-			b += fd->size;
+			if (fd) b += fd->size;
 			}
 
 		*bytes = b;
@@ -915,6 +915,8 @@ void vflist_select_by_path(ViewFileList *vfl, const gchar *path)
  *-----------------------------------------------------------------------------
  */
 
+#define CLIST_FRAME_SIZE 4
+
 static void vflist_populate_view(ViewFileList *vfl)
 {
 	GtkCList *clist;
@@ -923,9 +925,12 @@ static void vflist_populate_view(ViewFileList *vfl)
 	GList *work;
 	gint width;
 	gint tmp_width;
-	gint row;
+#if 0
 	const gchar *image_name = NULL;
 	gint image_row = -1;
+#endif
+	gint visible_row = -1;
+	gfloat visible_offset = 0.0;
 	gint row_p = 0;
 
 
@@ -941,6 +946,20 @@ static void vflist_populate_view(ViewFileList *vfl)
 		return;
 		}
 
+	gtk_clist_get_selection_info(clist, 0, 0, &visible_row, NULL);
+	if (clist->row_height > 0 && clist->clist_window_height > 0 && visible_row < clist->rows - 1)
+		{
+		gint icon_offset;
+
+		/* ugly, but it maintains exact scroll offset when refreshing */
+
+		icon_offset = clist->row_height + (clist->voffset % (clist->row_height + 1));
+		visible_offset = (gfloat)icon_offset / (clist->clist_window_height - clist->row_height - CLIST_FRAME_SIZE);
+
+		visible_row++;
+		}
+
+#if 0
 	if (vfl->layout)
 		{
 		gchar *buf;
@@ -952,7 +971,9 @@ static void vflist_populate_view(ViewFileList *vfl)
 			}
 		g_free(buf);
 		}
-	else
+#endif
+
+	
 
 	gtk_clist_freeze(clist);
 
@@ -994,6 +1015,7 @@ static void vflist_populate_view(ViewFileList *vfl)
 
 			if (match < 0)
 				{
+				gint row;
 				gchar *buf[4];
 
 				buf[0] = (gchar *)fd->name;
@@ -1008,16 +1030,20 @@ static void vflist_populate_view(ViewFileList *vfl)
 					{
 					gtk_clist_set_shift(clist, row, 0, 0, 5 + thumb_max_width);
 					}
+
 				done = TRUE;
+				if (row_p <= visible_row) visible_row++;
+#if 0
 				if (image_name && strcmp(fd->name, image_name) == 0)
 					{
-					gtk_clist_select_row(clist, row, 0);
 					image_row = row;
 					}
+#endif
 				}
 			else if (match > 0)
 				{
 				gtk_clist_remove(clist, row_p);
+				if (row_p < visible_row) visible_row--;
 				}
 			else
 				{
@@ -1052,7 +1078,14 @@ static void vflist_populate_view(ViewFileList *vfl)
 					}
 
 				gtk_clist_set_row_data(clist, row_p, fd);
+
 				done = TRUE;
+#if 0
+				if (image_name && strcmp(fd->name, image_name) == 0)
+					{
+					image_row = row_p;
+					}
+#endif
 				}
 			}
 		row_p++;
@@ -1071,13 +1104,24 @@ static void vflist_populate_view(ViewFileList *vfl)
 
 	gtk_clist_set_column_width(clist, 0, width);
 
-	if (image_row != -1 && gtk_clist_row_is_visible(clist, image_row) == GTK_VISIBILITY_NONE)
-		{
-		gtk_clist_moveto(clist, image_row, 0, 0.50, 0.0);
+#if 0
+	if (image_row >= 0)
+	       	{
+		gtk_clist_select_row(clist, image_row, 0);
+		if (gtk_clist_row_is_visible(clist, image_row) != GTK_VISIBILITY_FULL)
+			{
+			gtk_clist_moveto(clist, image_row, 0, 0.50, 0.0);
+			}
 		}
+#endif
 
 	gtk_clist_thaw(clist);
 
+	if (visible_row > 0)
+		{
+		gtk_clist_moveto(clist, visible_row, -1, visible_offset, 0.0);
+		}
+
 	vflist_send_update(vfl);
 	vflist_thumb_update(vfl);
 }
@@ -1356,7 +1400,8 @@ void vflist_maint_removed(ViewFileList *vfl, const gchar *path, GList *ignore_li
 	row = vflist_index_by_path(vfl, path);
 	if (row < 0) return;
 
-	if (vflist_index_is_selected(vfl, row))
+	if (vflist_index_is_selected(vfl, row) &&
+	    layout_image_get_collection(vfl->layout, NULL) == NULL)
 		{
 		gint n;
 
